{-|
Module      : Gargantext.Core.Worker.Broker
Description : Broker utilities
Copyright   : (c) CNRS, 2017-Present
License     : AGPL + CECILL v3
Maintainer  : team@gargantext.org
Stability   : experimental
Portability : POSIX

-}

module Gargantext.Core.Worker.Broker
  ( initBrokerWithDBCreate )
where

import Async.Worker.Broker.PGMQ (PGMQBroker, BrokerInitParams(PGMQBrokerInitParams))
import Async.Worker.Broker.Types (Broker, initBroker)
import Async.Worker.Types qualified as W
import Data.Text qualified as T
import Data.Text.Encoding qualified as TE
import Database.PostgreSQL.Simple qualified as PSQL
import Gargantext.Core.Config (GargConfig(..), gc_worker)
import Gargantext.Core.Config.Worker (WorkerSettings(..))
import Gargantext.Core.Worker.Jobs.Types (Job(..))
import Gargantext.Database.Prelude (createDBIfNotExists)
import Gargantext.Prelude



-- | Create DB if not exists, then run 'initBroker' (which, in
-- particular, creates the pgmq extension, if needed)
initBrokerWithDBCreate :: (W.HasWorkerBroker PGMQBroker Job)
                       => GargConfig
                       -> IO (Broker PGMQBroker (W.Job Job))
initBrokerWithDBCreate gc@(GargConfig { _gc_database_config }) = do
  -- By using gargantext db credentials, we create pgmq db (if needed)
  let WorkerSettings { .. }  = gc ^. gc_worker
  let psqlDB = TE.decodeUtf8 $ PSQL.postgreSQLConnectionString _gc_database_config

  createDBIfNotExists psqlDB (T.pack $ PSQL.connectDatabase _wsDatabase)
  
  initBroker $ PGMQBrokerInitParams _wsDatabase _wsDefaultVisibilityTimeout
